/*
 * Physically Based Simulation, SS 2005
 *
 * Markus Liechti (liechtim@student.ethz.ch)
 * Yoshimi Takano (yoshi@student.ethz.ch)
 * Stefan Wismer (swismer@student.ethz.ch)
 *
 * $Id: main.cpp,v 1.4 2005/06/26 00:51:08 ytakano Exp $
 */

#include <ctime>
#include <cstdlib>
#include "world.h"
#include "bridgeGUI.h"

#ifdef WIN32
	#define EXE "S12.exe"
#else
	#if defined(__APPLE__) && defined(__MACH__)
		#define EXE "S12.macosx"
	#else
		#define EXE "S12"
	#endif
#endif

using namespace std;

void printUsage(ostream &os) {
	os << "Usage: " << EXE << " [-b <bridgeFile>] [-t <trainFile>] [-f]" << endl;
	os << "Options:" << endl;
	os << "  -b <bridgeFile> use bridge described in <bridgeFile> instead of default bridge" << endl;
	os << "  -t <trainFile>  use train described in <trainFile> instead of default train" << endl;
	os << "  -f              start in full screen mode" << endl;
	exit(1);
}

void parseCommandLine(int argc, char **argv, char **bridgeFile, char **trainFile, bool *fullScreen) {
	*bridgeFile = NULL;
	*trainFile = NULL;
	*fullScreen = false;
    
	for (int i = 1; i < argc; i++) {
		if (strcmp("-b", argv[i]) == 0) {
			if (argc > i + 1) {
				*bridgeFile = argv[++i];
			} else {
				printUsage(cerr);
			}
		} else if (strcmp("-t", argv[i]) == 0) {
			if (argc > i + 1) {
				*trainFile = argv[++i];
			} else {
				printUsage(cerr);
			}
		} else if (strcmp("-f", argv[i]) == 0) {
			*fullScreen = true;
		} else {
			printUsage(cerr);
		}
	}
}

int main(int argc, char **argv) {
	char *bridgeFile, *trainFile;
	bool fullScreen;

	parseCommandLine(argc, argv, &bridgeFile, &trainFile, &fullScreen);

	srand(time(0)); // initialize random number generator
	World world;
	BridgeGUI gui(&world);
	gui.run(800, 600, fullScreen, bridgeFile, trainFile);

	return 0;
}
