/*
 * Physically Based Simulation, SS 2005
 *
 * Markus Liechti (liechtim@student.ethz.ch)
 * Yoshimi Takano (yoshi@student.ethz.ch)
 * Stefan Wismer (swismer@student.ethz.ch)
 *
 * $Id: particle.h,v 1.6 2005/06/26 20:21:19 ytakano Exp $
 */

#ifndef __particle_H__
#define __particle_H__

#include "vector2.h"

/**
 * Represents a particle of a particle system.
 */
struct Particle {
	/** Constructs an uninitialized particle. */
	Particle() {}

	/**
	 * Constructs a particle.
	 * @param m the mass
	 * @param p the position
	 * @param v the velocity
	 * @param l the life time
	 */
	Particle(component m, vector2 p, vector2 v, component l):
		age(0.0), lifetime(l), mass(m), position(p), velocity(v) {}

	/** The current age. */
	component age;

	/** The life time. */
	component lifetime;

	/** The mass. */
	component mass;

	/** The current position. */
	vector2 position;

	/** The current velocity. */
	vector2 velocity;

	/** The current force. */
	vector2 force;
};

#endif
