/*
 * Physically Based Simulation, SS 2005
 *
 * Markus Liechti (liechtim@student.ethz.ch)
 * Yoshimi Takano (yoshi@student.ethz.ch)
 * Stefan Wismer (swismer@student.ethz.ch)
 *
 * $Id: puff.cpp,v 1.4 2005/06/26 13:02:58 ytakano Exp $
 */

#include "puff.h"

void Puff::clear() {
	particles.clear();
}

void Puff::createPuff(vector2 &start, vector2 &end) {
	vector2 velMean(PUFF_VEL_MEAN_X, PUFF_VEL_MEAN_Y);
	vector2 d = end - start;
	component l = length(d);
	int puffs = PUFF_PARTICLES_PER_LENGTH_UNIT * static_cast<int>(l);
	vector2 position = start;
	vector2 delta = 1.0/static_cast<component>(puffs - 1) * d;
	for (int i = 0; i < puffs; i++, position += delta) {
		component mass = perturbate(PUFF_MASS_MEAN, PUFF_MASS_STD);
		vector2 pos = perturbate(position, PUFF_POS_STD);
		vector2 vel = perturbate(velMean, PUFF_VEL_STD);
		component lifetime = perturbate(PUFF_LIEFTIME_MEAN, PUFF_LIEFTIME_STD);

		particles.push_back(Particle(mass, pos, vel, lifetime));
	}
}

void Puff::setExternalForces() {
	vector2 forceMean(PUFF_FORCE_MEAN_X, PUFF_FORCE_MEAN_Y);
	for (list<Particle>::iterator i = particles.begin(); i != particles.end(); i++) {
		i->force = perturbate(forceMean, PUFF_FORCE_STD);
	}
}

void Puff::doPhysics(component timestep) {
	setExternalForces();

	list<Particle>::iterator i = particles.begin();
	while (i != particles.end()) {
		// leap frog integration
		vector2 a = (i->force - PUFF_DAMPING * i->velocity) / i->mass;
		i->velocity += timestep * a;
		i->position += timestep * i->velocity;
		i->age += timestep;

		// check age
		if (i->age > i->lifetime) {
			i = particles.erase(i);
		} else {
			i++;
		}
	}
}

void Puff::draw() {
	glPointSize(1.0);
	glColor3f(0.8, 0.7, 0.7);
	glBegin(GL_POINTS);
	for (list<Particle>::iterator i = particles.begin(); i != particles.end(); i++) {
		if (i->age <= i->lifetime) {
			glVertex3f(i->position.x, i->position.y, 2.0); // make the particles appear in the front
		}
	}
	glEnd();
}
