/*
 * Physically Based Simulation, SS 2005
 *
 * Markus Liechti (liechtim@student.ethz.ch)
 * Yoshimi Takano (yoshi@student.ethz.ch)
 * Stefan Wismer (swismer@student.ethz.ch)
 *
 * $Id: puff.h,v 1.6 2005/06/26 18:40:25 ytakano Exp $
 */

 /** @file */

#ifndef __puff_H__
#define __puff_H__

#include <list>
#include "opengl.h"
#include "particle.h"

/** The number of particles per length unit of a (broken) edge. */
#define PUFF_PARTICLES_PER_LENGTH_UNIT 10

/** The damping coefficient. */
#define PUFF_DAMPING 0.02

/** The mean mass. */
#define PUFF_MASS_MEAN 0.01

/** The standard deviation of the mass. */
#define PUFF_MASS_STD 0.001

/** The standard deviation of the initial position. */
#define PUFF_POS_STD 0.1

/** The mean initial velocity (x-component). */
#define PUFF_VEL_MEAN_X 0.0

/** The mean initial velocity (y-component). */
#define PUFF_VEL_MEAN_Y 2.0

/** The standard deviation of the initial velocity. */
#define PUFF_VEL_STD 0.1

/** The mean lifetime. */
#define PUFF_LIEFTIME_MEAN 0.5

/** The standard deviation of the lifetime. */
#define PUFF_LIEFTIME_STD 0.1

/** The mean external force (x-component). */
#define PUFF_FORCE_MEAN_X 0.0

/** The mean external force (y-component). */
#define PUFF_FORCE_MEAN_Y -0.2

/** The standard deviation of the external force. */
#define PUFF_FORCE_STD 0.001

using namespace std;

/**
 * A particle system to simulate "puffs" in the event of a broken edge.
 */
class Puff {
private:
	list<Particle> particles;
	void setExternalForces();
	
public:
	/**
	 * Clears all particles.
	 */
	void clear();

	/**
	 * Constructs a "puff" over a (broken) edge. The particles are
	 * distributed randomly along the edge.
	 * @param start the start point of the edge
	 * @param end the end point of the edge
	 */
	void createPuff(vector2 &start, vector2 &end);

	/**
	 * Performs a step in the puff particle simulation.
	 * @param timestep the time step
	 */
	void doPhysics(component timestep);

	/**
	 * Draws the puff particles.
	 */
	void draw();
};

#endif
