/*
 * Physically Based Simulation, SS 2005
 *
 * Markus Liechti (liechtim@student.ethz.ch)
 * Yoshimi Takano (yoshi@student.ethz.ch)
 * Stefan Wismer (swismer@student.ethz.ch)
 *
 * $Id: smoke.h,v 1.11 2005/06/25 20:23:44 ytakano Exp $
 */

/** @file */

#ifndef __smoke_H__
#define __smoke_H__

#include <list>
#include "opengl.h"
#include "particle.h"

/** The path to the smoke texture bitmap. */
#define SMOKE "data/smoke.bmp"

/** The path to the transparency mask bitmap. */
#define SMOKEMASK "data/smokemask.bmp"

/** The damping coefficient. */
#define SMOKE_DAMPING 0.04

/** The mean mass. */
#define SMOKE_MASS_MEAN 0.01

/** The standard deviation of the mass. */
#define SMOKE_MASS_STD 0.001

/** The standard deviation of the initial position. */
#define SMOKE_POS_STD 0.1

/** The standard deviation of the initial velocity length. */
#define SMOKE_VEL_LENGTH_STD 1.0

/** The mean rotation angle of the initial velocity. */
#define SMOKE_VEL_ANGLE_MEAN 0.0

/** The standard deviation of rotation angle of the initial velocity. */
#define SMOKE_VEL_ANGLE_STD 0.1

/** The mean lifetime. */
#define SMOKE_LIEFTIME_MEAN 2.0

/** The standard deviation of the lifetime. */
#define SMOKE_LIEFTIME_STD 1.0

/** The mean external force (x-component). */
#define SMOKE_FORCE_MEAN_X 0.0

/** The mean external force (y-component). */
#define SMOKE_FORCE_MEAN_Y 0.15 // points upward

/** The standard deviation of the external force. */
#define SMOKE_FORCE_STD 0.001

/**
 * Determines the decrement factor of the external force, depending on the
 * relative particle age. The factor
 * <tt>(SMOKE_FORCE_AGE_COEFF - relative age)</tt>
 * is applied to the (perturbated) external force in each time step.
 */
#define SMOKE_FORCE_AGE_COEFF 0.5

using namespace std;

/**
 * A particle system to simulate smoke.
 */
class Smoke {
private:
	list<Particle> particles;
	void setExternalForces();
	
	SDL_Surface *texture, *mask;

public:
	/** The dimensions of the texture image. */
	int textureWidth, textureHeight;

	/** The texture image. */
	unsigned char *image;

	/**
	 * Constructor. Loads all needed textures.
	 */
	 Smoke();

	/**
	 * Clears all particles.
	 */
	void clear();

	/**
	 * Creates new smoke particles.
	 * @param start the initial position of the particles (perturbated for each
	 * particle)
	 * @param velocity the initial velocity of the particles (perturbated for
	 * ecah particle)
	 * @param n the number of particles to be created
	 */
	void createSmoke(vector2 &start, vector2 &velocity, int n);

	/**
	 * Performs a step in the smoke particle simulation.
	 * @param timestep the time step
	 */
	void doPhysics(component timestep);

	/**
	 * Draws the smoke particles.
	 */
	void draw();
};

#endif
