/*
 * Physically Based Simulation, SS 2005
 *
 * Markus Liechti (liechtim@student.ethz.ch)
 * Yoshimi Takano (yoshi@student.ethz.ch)
 * Stefan Wismer (swismer@student.ethz.ch)
 *
 * $Id: world.cpp,v 1.10 2005/06/25 19:58:24 ytakano Exp $
 */

#include <vector>
#include <iostream>
#include <fstream>
#include "world.h"

using namespace std;

bool World::saveToFile(char *filename) {
	ofstream fout(filename, ios::out);
	if (!fout.is_open()) {
		return false;
	}

	// write horizon
	fout << horizon.size() << endl;
	for (vector<vertex>::iterator i = horizon.begin(); i != horizon.end(); i++) {
		fout << (int) i->initialPosition.x  << " " << (int) i->initialPosition.y << endl;
	}
	// write vertices
	fout << bridge.vertices.size() << endl;
	for (vector<vertex>::iterator i = bridge.vertices.begin(); i != bridge.vertices.end(); i++) {
		fout << (int) i->initialPosition.x  << " " << (int) i->initialPosition.y << " " << i->type << endl;
	}
	// write edges
	fout << bridge.edges.size() << endl;
	for (vector<edge>::iterator i = bridge.edges.begin(); i != bridge.edges.end(); i++) {
		fout << i->start  << " " << i->end << endl;
	}
	fout.close();
	return true;
}

bool World::loadFromFile(char *filename) {
	ifstream fin;
	fin.open(filename, ios::in);
	if (!fin.is_open()) {
		return false;
	}
	bridge.vertices.clear();
	bridge.edges.clear();
	horizon.clear();
	
	int size;
	vertex tmpV;
	edge tmpE;
	
	//read horizon
	fin >> size;
	tmpV.type = vertex::fixed; // just an arbitrary value
	for (int i = 0; i < size; i++) {
		fin >> tmpV.initialPosition.x >> tmpV.initialPosition.y;
		horizon.push_back(tmpV);
	}
	// read vertices
	fin >> size;
	for (int i = 0; i < size; i++) {
		int type;
		fin >> tmpV.initialPosition.x >> tmpV.initialPosition.y >> type;
		tmpV.type = static_cast<vertex::vertexType>(type);
		bridge.vertices.push_back(tmpV);
	}
	// read edges
	fin >> size;
	for (int i = 0; i < size; i++) {
		fin >> tmpE.start >> tmpE.end;
		bridge.edges.push_back(tmpE);
	}
	fin.close();
	return true;
}
